@@ -1,6 +1,6 @@
 {-# LANGUAGE BangPatterns, CPP, DeriveDataTypeable, FlexibleContexts,
     FlexibleInstances, MultiParamTypeClasses, MagicHash, Rank2Types,
-    ScopedTypeVariables, TypeFamilies, UnboxedTuples
+    ScopedTypeVariables, TypeFamilies, UnboxedTuples, TypeOperators
     #-}
 -- |
 -- Module    : System.Random.MWC
@@ -15,7 +15,7 @@
 -- known as MWC8222) multiply-with-carry generator, which has a period
 -- of \(2^{8222}\) and fares well in tests of randomness.  It is also
 -- extremely fast, between 2 and 3 times faster than the Mersenne
--- Twister. There're two representation of generator: 'Gen' which is
+-- Twister. There are two representation of generator: 'Gen' which is
 -- generator that uses in-place mutation and 'Seed' which is immutable
 -- snapshot of generator's state.
 --
@@ -75,7 +75,7 @@
 -- >>> replicateM 10 $ uniformM g :: IO [Word8]
 -- [209,138,126,150,165,15,69,203,155,146]
 --
--- There're special functions for generation of @Doubles@ and @Float
+-- There are special functions for generation of @Doubles@ and @Float
 -- in unit interval: 'Random.uniformDouble01M',
 -- 'Random.uniformDoublePositive01M', 'Random.uniformFloat01M',
 -- 'Random.uniformFloatPositive01M':
@@ -104,7 +104,7 @@
 -- [138,242,130,33,209,248,89,134,150,180]
 --
 -- Note that 'Random.uniformByteStringM' produces different result
--- from 'uniformVector' since it uses PRNG's output more efficently.
+-- from 'uniformVector' since it uses PRNG's output more efficiently.
 --
 --
 -- == State handling
@@ -159,7 +159,7 @@ module System.Random.MWC
 #endif
 
 import Control.Monad           (ap, liftM, unless)
-import Control.Monad.Primitive (PrimMonad, PrimBase, PrimState, unsafePrimToIO, unsafeSTToPrim)
+import Control.Monad.Primitive (PrimMonad, PrimBase, PrimState, unsafePrimToIO, stToPrim)
 import Control.Monad.ST        (ST,runST)
 import Data.Bits               ((.&.), (.|.), shiftL, shiftR, xor)
 import Data.Int                (Int8, Int16, Int32, Int64)
@@ -167,6 +167,7 @@ import Data.IORef              (IORef, atomicModifyIORef, newIORef)
 import Data.Typeable           (Typeable)
 import Data.Vector.Generic     (Vector)
 import Data.Word
+import Data.Kind
 import qualified Data.Vector.Generic         as G
 import qualified Data.Vector.Generic.Mutable as GM
 import qualified Data.Vector.Unboxed         as I
@@ -462,7 +463,7 @@ instance (s ~ PrimState m, PrimMonad m) => Random.StatefulGen (Gen s) m where
   {-# INLINE uniformWord32 #-}
   uniformWord64 = uniform
   {-# INLINE uniformWord64 #-}
-  uniformShortByteString n g = unsafeSTToPrim (Random.genShortByteStringST n (uniform g))
+  uniformShortByteString n g = stToPrim (Random.genShortByteStringST n (uniform g))
   {-# INLINE uniformShortByteString #-}
 
 -- | @since 0.15.0.0
@@ -471,7 +472,7 @@ instance PrimMonad m => Random.FrozenGen Seed m where
   thawGen = restore
   freezeGen = save
 
--- | Convert vector to 'Seed'. It acts similarily to 'initialize' and
+-- | Convert vector to 'Seed'. It acts similarly to 'initialize' and
 -- will accept any vector. If you want to pass seed immediately to
 -- restore you better call initialize directly since following law holds:
 --
@@ -621,9 +622,9 @@ uniform2 f (Gen q) = do
 {-# INLINE uniform2 #-}
 
 -- Type family for fixed size integrals. For signed data types it's
--- its unsigned couterpart with same size and for unsigned data types
+-- its unsigned counterpart with same size and for unsigned data types
 -- it's same type
-type family Unsigned a :: *
+type family Unsigned a :: Type
 
 type instance Unsigned Int8  = Word8
 type instance Unsigned Int16 = Word16
@@ -813,7 +814,7 @@ defaultSeed = I.fromList [
 -- > if the carry value is strictly smaller than the multiplicator,
 -- > the next carry value is also strictly smaller than the multiplicator.
 --
--- Eventhough the proof is written in terms of the actual value of the
+-- Even though the proof is written in terms of the actual value of the
 -- multiplicator, it holds for any multiplicator value /not/ greater
 -- than maxBound 'Word32'
 --
