@@ -1,3 +1,4 @@
+{-# LANGUAGE MultiWayIf #-}
 {-# LANGUAGE BangPatterns, CPP, GADTs, FlexibleContexts, ScopedTypeVariables #-}
 -- |
 -- Module    : System.Random.MWC.Distributions
@@ -27,6 +28,7 @@ module System.Random.MWC.Distributions
     , geometric0
     , geometric1
     , bernoulli
+    , binomial
       -- ** Multivariate
     , dirichlet
       -- * Permutations
@@ -47,10 +49,11 @@ import Data.Traversable (Traversable)
 #endif
 import Data.Traversable (mapM)
 import Data.Word (Word32)
-import System.Random.MWC (Gen, uniform, uniformR)
+import System.Random.Stateful (StatefulGen(..),Uniform(..),UniformRange(..),uniformDoublePositive01M)
 import qualified Data.Vector.Unboxed         as I
 import qualified Data.Vector.Generic         as G
 import qualified Data.Vector.Generic.Mutable as M
+import Numeric.SpecFunctions (logFactorial)
 
 -- Unboxed 2-tuple
 data T = T {-# UNPACK #-} !Double {-# UNPACK #-} !Double
@@ -58,10 +61,10 @@ data T = T {-# UNPACK #-} !Double {-# UNPACK #-} !Double
 
 -- | Generate a normally distributed random variate with given mean
 -- and standard deviation.
-normal :: PrimMonad m
+normal :: StatefulGen g m
        => Double                -- ^ Mean
        -> Double                -- ^ Standard deviation
-       -> Gen (PrimState m)
+       -> g
        -> m Double
 {-# INLINE normal #-}
 normal m s gen = do
@@ -75,13 +78,13 @@ normal m s gen = do
 -- Compared to the ziggurat algorithm usually used, this is slower,
 -- but generates more independent variates that pass stringent tests
 -- of randomness.
-standard :: PrimMonad m => Gen (PrimState m) -> m Double
+standard :: StatefulGen g m => g -> m Double
 {-# INLINE standard #-}
 standard gen = loop
   where
     loop = do
-      u  <- (subtract 1 . (*2)) `liftM` uniform gen
-      ri <- uniform gen
+      u  <- (subtract 1 . (*2)) `liftM` uniformDoublePositive01M gen
+      ri <- uniformM gen
       let i  = fromIntegral ((ri :: Word32) .&. 127)
           bi = I.unsafeIndex blocks i
           bj = I.unsafeIndex blocks (i+1)
@@ -93,14 +96,14 @@ standard gen = loop
                  xx = x * x
                  d  = exp (-0.5 * (bi * bi - xx))
                  e  = exp (-0.5 * (bj * bj - xx))
-             c <- uniform gen
+             c <- uniformDoublePositive01M gen
              if e + c * (d - e) < 1
                then return x
                else loop
     normalTail neg  = tailing
       where tailing  = do
-              x <- ((/rNorm) . log) `liftM` uniform gen
-              y <- log              `liftM` uniform gen
+              x <- ((/rNorm) . log) `liftM` uniformDoublePositive01M gen
+              y <- log              `liftM` uniformDoublePositive01M gen
               if y * (-2) < x * x
                 then tailing
                 else return $! if neg then x - rNorm else rNorm - x
@@ -129,36 +132,36 @@ ratios = I.zipWith (/) (I.tail blocks) blocks
 
 
 -- | Generate an exponentially distributed random variate.
-exponential :: PrimMonad m
+exponential :: StatefulGen g m
             => Double            -- ^ Scale parameter
-            -> Gen (PrimState m) -- ^ Generator
+            -> g                 -- ^ Generator
             -> m Double
 {-# INLINE exponential #-}
 exponential b gen = do
-  x <- uniform gen
+  x <- uniformDoublePositive01M gen
   return $! - log x / b
 
 
 -- | Generate truncated exponentially distributed random variate.
-truncatedExp :: PrimMonad m
+truncatedExp :: StatefulGen g m
              => Double            -- ^ Scale parameter
              -> (Double,Double)   -- ^ Range to which distribution is
                                   --   truncated. Values may be negative.
-             -> Gen (PrimState m) -- ^ Generator.
+             -> g                 -- ^ Generator.
              -> m Double
 {-# INLINE truncatedExp #-}
 truncatedExp scale (a,b) gen = do
   -- We shift a to 0 and then generate distribution truncated to [0,b-a]
   -- It's easier
   let delta = b - a
-  p <- uniform gen
+  p <- uniformDoublePositive01M gen
   return $! a - log ( (1 - p) + p*exp(-scale*delta)) / scale
 
 -- | Random variate generator for gamma distribution.
-gamma :: PrimMonad m
+gamma :: (StatefulGen g m)
       => Double                 -- ^ Shape parameter
       -> Double                 -- ^ Scale parameter
-      -> Gen (PrimState m)      -- ^ Generator
+      -> g                      -- ^ Generator
       -> m Double
 {-# INLINE gamma #-}
 gamma a b gen
@@ -167,13 +170,13 @@ gamma a b gen
     where
       mainloop = do
         T x v <- innerloop
-        u     <- uniform gen
+        u     <- uniformDoublePositive01M gen
         let cont =  u > 1 - 0.331 * sqr (sqr x)
                  && log u > 0.5 * sqr x + a1 * (1 - v + log v) -- Rarely evaluated
         case () of
           _| cont      -> mainloop
            | a >= 1    -> return $! a1 * v * b
-           | otherwise -> do y <- uniform gen
+           | otherwise -> do y <- uniformDoublePositive01M gen
                              return $! y ** (1 / a) * a1 * v * b
       -- inner loop
       innerloop = do
@@ -188,9 +191,9 @@ gamma a b gen
 
 
 -- | Random variate generator for the chi square distribution.
-chiSquare :: PrimMonad m
+chiSquare :: StatefulGen g m
           => Int                -- ^ Number of degrees of freedom
-          -> Gen (PrimState m)  -- ^ Generator
+          -> g                  -- ^ Generator
           -> m Double
 {-# INLINE chiSquare #-}
 chiSquare n gen
@@ -201,14 +204,14 @@ chiSquare n gen
 -- | Random variate generator for the geometric distribution,
 -- computing the number of failures before success. Distribution's
 -- support is [0..].
-geometric0 :: PrimMonad m
+geometric0 :: StatefulGen g m
            => Double            -- ^ /p/ success probability lies in (0,1]
-           -> Gen (PrimState m) -- ^ Generator
+           -> g                 -- ^ Generator
            -> m Int
 {-# INLINE geometric0 #-}
 geometric0 p gen
   | p == 1          = return 0
-  | p >  0 && p < 1 = do q <- uniform gen
+  | p >  0 && p < 1 = do q <- uniformDoublePositive01M gen
                          -- FIXME: We want to use log1p here but it will
                          --        introduce dependency on math-functions.
                          return $! floor $ log q / log (1 - p)
@@ -217,19 +220,19 @@ geometric0 p gen
 -- | Random variate generator for geometric distribution for number of
 -- trials. Distribution's support is [1..] (i.e. just 'geometric0'
 -- shifted by 1).
-geometric1 :: PrimMonad m
+geometric1 :: StatefulGen g m
            => Double            -- ^ /p/ success probability lies in (0,1]
-           -> Gen (PrimState m) -- ^ Generator
+           -> g                 -- ^ Generator
            -> m Int
 {-# INLINE geometric1 #-}
 geometric1 p gen = do n <- geometric0 p gen
                       return $! n + 1
 
 -- | Random variate generator for Beta distribution
-beta :: PrimMonad m
+beta :: StatefulGen g m
      => Double            -- ^ alpha (>0)
      -> Double            -- ^ beta  (>0)
-     -> Gen (PrimState m) -- ^ Generator
+     -> g                 -- ^ Generator
      -> m Double
 {-# INLINE beta #-}
 beta a b gen = do
@@ -238,9 +241,9 @@ beta a b gen = do
   return $! x / (x+y)
 
 -- | Random variate generator for Dirichlet distribution
-dirichlet :: (PrimMonad m, Traversable t)
+dirichlet :: (StatefulGen g m, Traversable t)
           => t Double          -- ^ container of parameters
-          -> Gen (PrimState m) -- ^ Generator
+          -> g                 -- ^ Generator
           -> m (t Double)
 {-# INLINE dirichlet #-}
 dirichlet t gen = do
@@ -249,12 +252,12 @@ dirichlet t gen = do
   return $ fmap (/total) t'
 
 -- | Random variate generator for Bernoulli distribution
-bernoulli :: PrimMonad m
+bernoulli :: StatefulGen g m
           => Double            -- ^ Probability of success (returning True)
-          -> Gen (PrimState m) -- ^ Generator
+          -> g                 -- ^ Generator
           -> m Bool
 {-# INLINE bernoulli #-}
-bernoulli p gen = (<p) `liftM` uniform gen
+bernoulli p gen = (<p) `liftM` uniformDoublePositive01M gen
 
 -- | Random variate generator for categorical distribution.
 --
@@ -262,16 +265,16 @@ bernoulli p gen = (<p) `liftM` uniform gen
 --   "System.Random.MWC.CondensedTable" will offer better
 --   performance.  If only few is needed this function will faster
 --   since it avoids costs of setting up table.
-categorical :: (PrimMonad m, G.Vector v Double)
+categorical :: (StatefulGen g m, G.Vector v Double)
             => v Double          -- ^ List of weights [>0]
-            -> Gen (PrimState m) -- ^ Generator
+            -> g                 -- ^ Generator
             -> m Int
 {-# INLINE categorical #-}
 categorical v gen
     | G.null v = pkgError "categorical" "empty weights!"
     | otherwise = do
         let cv  = G.scanl1' (+) v
-        p <- (G.last cv *) `liftM` uniform gen
+        p <- (G.last cv *) `liftM` uniformDoublePositive01M gen
         return $! case G.findIndex (>=p) cv of
                     Just i  -> i
                     Nothing -> pkgError "categorical" "bad weights!"
@@ -279,9 +282,9 @@ categorical v gen
 -- | Random variate generator for categorical distribution where the
 --   weights are in the log domain. It's implemented in terms of
 --   'categorical'.
-logCategorical :: (PrimMonad m, G.Vector v Double)
+logCategorical :: (StatefulGen g m, G.Vector v Double)
                => v Double          -- ^ List of logarithms of weights
-               -> Gen (PrimState m) -- ^ Generator
+               -> g                 -- ^ Generator
                -> m Int
 {-# INLINE logCategorical #-}
 logCategorical v gen
@@ -294,9 +297,9 @@ logCategorical v gen
 --   It returns random permutation of vector /[0 .. n-1]/.
 --
 --   This is the Fisher-Yates shuffle
-uniformPermutation :: forall m v. (PrimMonad m, G.Vector v Int)
+uniformPermutation :: forall g m v. (StatefulGen g m, PrimMonad m, G.Vector v Int)
                    => Int
-                   -> Gen (PrimState m)
+                   -> g
                    -> m (v Int)
 {-# INLINE uniformPermutation #-}
 uniformPermutation n gen
@@ -306,9 +309,9 @@ uniformPermutation n gen
 -- | Random variate generator for a uniformly distributed shuffle (all
 --   shuffles are equiprobable) of a vector. It uses Fisher-Yates
 --   shuffle algorithm.
-uniformShuffle :: (PrimMonad m, G.Vector v a)
+uniformShuffle :: (StatefulGen g m, PrimMonad m, G.Vector v a)
                => v a
-               -> Gen (PrimState m)
+               -> g
                -> m (v a)
 {-# INLINE uniformShuffle #-}
 uniformShuffle vec gen
@@ -320,9 +323,9 @@ uniformShuffle vec gen
 
 -- | In-place uniformly distributed shuffle (all shuffles are
 --   equiprobable)of a vector.
-uniformShuffleM :: (PrimMonad m, M.MVector v a)
+uniformShuffleM :: (StatefulGen g m, PrimMonad m, M.MVector v a)
                 => v (PrimState m) a
-                -> Gen (PrimState m)
+                -> g
                 -> m ()
 {-# INLINE uniformShuffleM #-}
 uniformShuffleM vec gen
@@ -332,7 +335,7 @@ uniformShuffleM vec gen
     n   = M.length vec
     lst = n-1
     loop i | i == lst  = return ()
-           | otherwise = do j <- uniformR (i,lst) gen
+           | otherwise = do j <- uniformRM (i,lst) gen
                             M.unsafeSwap vec i j
                             loop (i+1)
 
@@ -345,6 +348,144 @@ pkgError :: String -> String -> a
 pkgError func msg = error $ "System.Random.MWC.Distributions." ++ func ++
                             ": " ++ msg
 
+-- | Random variate generator for Binomial distribution. Will throw
+-- exception when parameters are out range.
+--
+-- The probability of getting exactly k successes in n trials is
+-- given by the probability mass function:
+--
+-- \[
+-- f(k;n,p) = \Pr(X = k) = \binom n k  p^k(1-p)^{n-k}
+-- \]
+binomial :: forall g m . StatefulGen g m
+         => Int               -- ^ Number of trials, must be positive.
+         -> Double            -- ^ Probability of success \(p \in [0,1]\)
+         -> g                 -- ^ Generator
+         -> m Int
+{-# INLINE binomial #-}
+binomial n p gen
+  | n <= 0             = pkgError "binomial" "number of trials must be positive"
+  | p < 0.0 || p > 1.0 = pkgError "binomial" "probability must be >= 0 and <= 1"
+  | p == 0.0 = return 0
+  | p == 1.0 = return n
+  | p <= 0.5 = if
+      | fromIntegral n * p < inv_thr -> binomialInv n p gen
+      | otherwise                    -> binomialTPE n p gen
+  | p > 0.5  = do
+      ix <- case 1 - p of
+        p' | fromIntegral n * p' < inv_thr -> binomialInv n p' gen
+           | otherwise                     -> binomialTPE n p' gen
+      pure $! n - ix
+    -- Reachable when p is NaN
+  | otherwise = pkgError "binomial" "probability must be >= 0 and <= 1"
+  where
+    -- Threshold for preferring the BINV algorithm / inverse cdf
+    -- logic. The paper suggests 10, Ranlib uses 30, R uses 30, Rust uses
+    -- 10 and GSL uses 14.
+    inv_thr = 10
+
+-- Binomial-Triangle-Parallelogram-Exponential algorithm (BTPE)
+-- described in Kachitvichyanukul1988
+binomialTPE :: forall g m . StatefulGen g m => Int -> Double -> g -> m Int
+{-# INLINE binomialTPE #-}
+binomialTPE n p g = loop
+  where
+    -- Main accept/reject loop
+    loop = do
+      u <- uniformRM (0.0, p4) g
+      v <- uniformDoublePositive01M g
+      selectArea u v
+    -- Acceptance / rejection of sample [step 5]
+    acceptReject :: Int -> Double -> m Int
+    acceptReject !ix !v
+      | var <= accept = return ix
+      | otherwise     = loop
+      where
+        var    = log v
+        accept = logFactorial bigM + logFactorial (n - bigM) -
+                 logFactorial ix - logFactorial (n - ix) +
+                 fromIntegral (ix - bigM) * log (p / q)
+    -- Select area to be used [Steps 1-4]
+    selectArea :: Double -> Double -> m Int
+    selectArea !u !v
+        -- Triangular region
+      | u <= p1 = return $! floor $ xm - p1 * v + u
+        -- Parallelogram region
+      | u <= p2 = do let x = xl + (u - p1) / c
+                         w = v * c + 1.0 - abs (x - xm) / p1
+                     if w > 1 || w <= 0
+                       then loop
+                       else do let ix = floor x
+                               acceptReject ix w
+        -- Left tail
+      | u <= p3 = case floor $ xl + log v / lambdaL of
+          ix | ix < 0    -> loop
+             | otherwise -> do let w = v * (u - p2) * lambdaL
+                               acceptReject ix w
+        -- Right tail
+      | otherwise = case floor $ xr - log v / lambdaR of
+          ix | ix > n    -> loop
+             | otherwise -> do let w = v * (u - p3) * lambdaR
+                               acceptReject ix w
+    ----------------------------------------
+    -- Constants used in algorithm. See [Step 0]
+    q    = 1 - p
+    np   = fromIntegral n * p
+    ffm  = np + p
+    bigM = floor ffm
+    -- Half integer mean (tip of triangle)
+    xm   = fromIntegral bigM + 0.5
+    -- p1: the distance to the left and right edges of the triangle
+    -- region below the target distribution; since height=1, also:
+    -- area of region (half base * height)
+    !p1 = let npq  = np * q
+          in fromIntegral (floor (2.195 * sqrt npq - 4.6 * q) :: Int) + 0.5
+    xl = xm - p1   -- Left edge of triangle
+    xr = xm + p1   -- Right edge of triangle
+    c  = 0.134 + 20.5 / (15.3 + fromIntegral bigM)
+    -- p1 + area of parallelogram region
+    !p2 = p1 * (1.0 + c + c)
+    --
+    lambdaL = let al = (ffm - xl) / (ffm - xl * p)
+              in al * (1.0 + 0.5 * al)
+    lambdaR = let ar = (xr - ffm) / (xr * q)
+              in ar * (1.0 + 0.5 * ar)
+    -- p2 + area of left tail
+    !p3 = p2 + c / lambdaL
+    -- p3 + area of right tail
+    !p4 = p3 + c / lambdaR
+
+
+-- Compute binomial variate using inversion method (BINV in
+-- Kachitvichyanukul1988)
+binomialInv :: StatefulGen g m => Int -> Double -> g -> m Int
+{-# INLINE binomialInv #-}
+binomialInv n p g = do
+  u <- uniformDoublePositive01M g
+  return $! invertBinomial n p u
+
+-- This function is defined on top level to avoid inlining it since it's rather
+-- large and we don't need specializations since it's monomorphic anyway
+invertBinomial
+  :: Int    -- N of trials
+  -> Double -- probability of success
+  -> Double -- Output of PRNG
+  -> Int
+invertBinomial !n !p !u0 = invert (q^n) u0 0
+  where
+    -- We forcing s&a in order to avoid allocating thunks. Those are
+    -- more expensive than computing them unconditionally
+    q  = 1 - p
+    !s = p / q
+    !a = fromIntegral (n + 1) * s
+    --
+    invert !r !u !x
+      | u <= r    = x
+      | otherwise = invert r' u' x'
+      where
+        u' = u - r
+        x' = x + 1
+        r' = r * ((a / fromIntegral x') - s)
 
 
 -- $references
@@ -357,3 +498,9 @@ pkgError func msg = error $ "System.Random.MWC.Distributions." ++ func ++
 --   (2007). Gaussian random number generators.
 --   /ACM Computing Surveys/ 39(4).
 --   <http://www.cse.cuhk.edu.hk/~phwl/mt/public/archives/papers/grng_acmcs07.pdf>
+--
+-- * Kachitvichyanukul, V. and Schmeiser, B. W.  Binomial Random
+--   Variate Generation.  Communications of the ACM, 31, 2 (February,
+--   1988) 216. <https://dl.acm.org/doi/pdf/10.1145/42372.42381>
+--   Here's an example of how the algorithm's sampling regions look
+--   ![Something](docs/RecreateFigure.svg)
